/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.routing.dto;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.tinelstudio.gis.common.dto.DefaultStreetDto;
import net.tinelstudio.gis.common.dto.StreetDto;

import org.junit.Test;

import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.PrecisionModel;
import com.vividsolutions.jts.io.WKTReader;

/**
 * @author TineL
 */
public class DefaultRouteTest {

  protected DefaultRoute createInstance() {
    return new DefaultRoute();
  }

  @Test
  public void testSetStreets() {
    List<StreetDto> streets=Collections.emptyList();
    DefaultRoute route=createInstance();
    route.setStreets(streets);
    assertSame(streets, route.getStreets());
  }

  @Test
  public void testSetCost() {
    double cost=Math.random()*100;
    DefaultRoute route=createInstance();
    route.setCost(cost);
    assertEquals(cost, route.getCost(), 0.0001);
  }

  @Test
  public void testSetTimeTaken() {
    long cost=(long)(Math.random()*10000);
    DefaultRoute route=createInstance();
    route.setTimeTaken(cost);
    assertEquals(cost, route.getTimeTaken());
  }

  @Test
  public void testMergeEmptyStreets() {
    List<StreetDto> streets=Collections.emptyList();
    DefaultRoute route=createInstance();
    route.setStreets(streets);
    LineString lines=route.mergeStreets();
    assertNull(lines);
  }

  @Test
  public void testMergeStreets() throws Exception {
    String lineString1="LINESTRING (14.514935 46.050266, 14.514571 46.049407)";
    String lineString2="LINESTRING (14.514571 46.049407, 14.514184 46.048507)";
    String lineString3="LINESTRING (14.514184 46.048507, 14.513777 46.047584)";
    String mergedLineString="LINESTRING (14.514935 46.050266, 14.514571 46.049407, 14.514184 46.048507, 14.513777 46.047584)";

    GeometryFactory gf=new GeometryFactory(new PrecisionModel(), 4326);
    WKTReader wktReader=new WKTReader(gf);
    LineString line1=(LineString)wktReader.read(lineString1);
    LineString line2=(LineString)wktReader.read(lineString2);
    LineString line3=(LineString)wktReader.read(lineString3);

    DefaultStreetDto street1=new DefaultStreetDto();
    street1.setLineString(line1);
    DefaultStreetDto street2=new DefaultStreetDto();
    street2.setLineString(line2);
    DefaultStreetDto street3=new DefaultStreetDto();
    street3.setLineString(line3);
    List<StreetDto> streets=new ArrayList<StreetDto>();
    Collections.addAll(streets, street1, street2, street3);

    DefaultRoute route=createInstance();
    route.setStreets(streets);
    LineString lines=route.mergeStreets();

    assertEquals(mergedLineString, lines.toString());
  }
}
